/*
 * $Header: /home/cvs/jakarta-slide/src/contrib/webdavgui/src/java/org/apache/webdav/ui/lib/methods/SPPutMethod.java,v 1.3 2001/10/10 04:42:45 msmith Exp $
 * $Revision: 1.3 $
 * $Date: 2001/10/10 04:42:45 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights 
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer. 
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:  
 *       "This product includes software developed by the 
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written 
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */ 

package org.apache.webdav.ui.lib.methods;

import java.io.*;

import org.apache.webdav.lib.*;
import org.apache.webdav.lib.methods.*;
import org.apache.webdav.ui.filechooser.dir.SPResourceNode;
import org.apache.webdav.ui.util.SPUtils;
import org.apache.webdav.ui.SPWebdavClient;

import org.apache.commons.httpclient.HttpMethod;
import org.apache.commons.httpclient.HttpClient;
import org.apache.commons.httpclient.State;

public class SPPutMethod extends PutMethod
      implements SPWebdavMethod {

  HttpClient client;
  private String lockToken=null;

  public SPPutMethod(HttpClient client)  {
    setClient(client);
  }

  public SPPutMethod(SPResourceNode resource) {
    setClient(resource.getClient());
    setPath(resource.getResourceUrlPath());
  }

  public void setClient(HttpClient client)  {
    this.client = client;
  }

  public HttpClient getClient() {
    return client;
  }

  public String getMessageContent()   {return null;}

  public void refresh() {
    if (hasBeenUsed())  {
      recycle();
    }
  }

  public SPWebdavMethod execute() throws Exception {
    lockToken = ((SPWebdavClient)client).getLockToken(path); //the current token from the server

    try {
      client.executeMethod(this.getWebdavMethodHandle());
      if (lockToken != null)  {
        SPUtils.lock(client, path); //by default, refresh the lock timeout
      }

    }
    catch(Exception e) {
      e.printStackTrace();
      throw e;
    }
    return this;
  }

  public HttpMethod getWebdavMethodHandle() {
    return this;
  }

  /**
   * Generate additional headers needed by the request.
   * Overwrites this inherited function by adding If header
   *
   * @param host the host
   * @param state State token
   */
  public void generateHeaders(String host, State state) {
    super.generateHeaders(host, state);
    if(lockToken != null)
        setHeader("If", "(<" + lockToken + ">)");
    else
        System.err.println("ERROR: lockToken is null!");
    setHeader("Content-Type", "text/xml");
  }

  public String toString()	{
    return (new String("PutMethod()"));
  }

}//End of SPPutMethod class

