/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/client/src/org/apache/webdav/lib/Ace.java,v 1.3 2001/09/23 13:14:10 dirkv Exp $
 * $Revision: 1.3 $
 * $Date: 2001/09/23 13:14:10 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.webdav.lib;

import java.util.Vector;
import java.util.Enumeration;

/**
 * This interface models a DAV Access control entry.
 *
 * @author Remy Maucherat
 * @author Dirk Verbeeck
 * @version $Revision: 1.3 $
 */
public class Ace {


    // ----------------------------------------------------------- Constructors


    public Ace(String principal) {
        this.principal = principal;
    }


    public Ace(String principal, boolean negative, boolean protectedAce,
               boolean inherited, String inheritedFrom) {
        this(principal);
        this.negative = negative;
        this.protectedAce = protectedAce;
        this.inherited = inherited;
        this.inheritedFrom = inheritedFrom;
    }


    // ----------------------------------------------------- Instance Variables


    /**
     * Principal.
     */
    protected String principal;


    /**
     * Negative (deny) flag.
     */
    protected boolean negative = false;


    /**
     * Privileges this ACE grants or denies.
     */
    protected Vector privileges = new Vector();


    /**
     * Protected.
     */
    protected boolean protectedAce = false;


    /**
     * Inherited.
     */
    protected boolean inherited = false;


    /**
     * Inherited from.
     */
    protected String inheritedFrom = null;


    // ------------------------------------------------------------- Properties


    /**
     * Principal accessor.
     */
    public String getPrincipal() {
        return principal;
    }


    /**
     * Principal mutator.
     */
    public void setPrincipal(String principal) {
        this.principal = principal;
    }


    /**
     * Negative accessor.
     */
    public boolean isNegative() {
        return (negative);
    }


    /**
     * Negative mutator.
     */
    public void setNegative(boolean negative) {
        this.negative = negative;
    }


    /**
     * Protected accessor.
     */
    public boolean isProtected() {
        return (protectedAce);
    }


    /**
     * Protected mutator.
     */
    public void setProtected(boolean protectedAce) {
        this.protectedAce = protectedAce;
    }


    /**
     * Inherited accessor.
     */
    public boolean isInherited() {
        return (inherited);
    }


    /**
     * Inherited mutator.
     */
    public void setInherited(boolean inherited) {
        this.inherited = inherited;
    }


    /**
     * Inherited from accessor.
     */
    public String getInheritedFrom() {
        return inheritedFrom;
    }


    /**
     * Inherited from mutator.
     */
    public void setInheritedFrom(String inheritedFrom) {
        this.inheritedFrom = inheritedFrom;
    }


    /**
     * Enumerate privileges.
     */
    public Enumeration enumeratePrivileges() {
        return privileges.elements();
    }


    /**
     * Add privilege.
     */
    public void addPrivilege(Privilege privilege) {
        privileges.addElement(privilege);
    }


    /**
     * Remove privilege.
     */
    public boolean removePrivilege(Privilege privilege) {
        return privileges.removeElement(privilege);
    }


    /**
     * Clear privileges.
     */
    public void clearPrivileges() {
        privileges.clear();
    }


	public String toString() {
		return ((!isNegative()?"granted":"denied") +
		" to " + getPrincipal() +
		" (" + (isProtected()?"protected":"not protected") + ")" +
		" (" + (isInherited()? ("inherited from '" + getInheritedFrom() + "'"): "not inherited") +")");
	}

}
