/*
 * $Header: /home/cvs/jakarta-slide/src/webdav/server/org/apache/slide/webdav/logger/XHttpServletRequestFacade23.java,v 1.2 2001/08/25 13:00:38 dirkv Exp $
 * $Revision: 1.2 $
 * $Date: 2001/08/25 13:00:38 $
 *
 * ====================================================================
 *
 * The Apache Software License, Version 1.1
 *
 * Copyright (c) 1999 The Apache Software Foundation.  All rights
 * reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 *
 * 3. The end-user documentation included with the redistribution, if
 *    any, must include the following acknowlegement:
 *       "This product includes software developed by the
 *        Apache Software Foundation (http://www.apache.org/)."
 *    Alternately, this acknowlegement may appear in the software itself,
 *    if and wherever such third-party acknowlegements normally appear.
 *
 * 4. The names "The Jakarta Project", "Tomcat", and "Apache Software
 *    Foundation" must not be used to endorse or promote products derived
 *    from this software without prior written permission. For written
 *    permission, please contact apache@apache.org.
 *
 * 5. Products derived from this software may not be called "Apache"
 *    nor may "Apache" appear in their names without prior written
 *    permission of the Apache Group.
 *
 * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
 * OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 * DISCLAIMED.  IN NO EVENT SHALL THE APACHE SOFTWARE FOUNDATION OR
 * ITS CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 * ====================================================================
 *
 * This software consists of voluntary contributions made by many
 * individuals on behalf of the Apache Software Foundation.  For more
 * information on the Apache Software Foundation, please see
 * <http://www.apache.org/>.
 *
 * [Additional notices, if required by prior licensing conditions]
 *
 */

package org.apache.slide.webdav.logger;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletRequestWrapper;
import javax.servlet.http.Cookie;
import javax.servlet.http.HttpSession;
import javax.servlet.ServletInputStream;
import javax.servlet.RequestDispatcher;

import java.io.InputStream;
import java.io.BufferedInputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.util.Enumeration;
import java.util.Locale;
import java.security.Principal;

import org.apache.slide.common.Domain;

/**
 * This class supports additional set-methods and a re-readable
 * inputstream to the interface javax.servlet.http.HttpServletRequest.
 *
 * @author    Hardy.Kiefer@softwareag.com
 *            Christopher Lenz (cmlenz at apache.org)
 *
 * @version   0.1
 *
 * @invariant (inputStream != null)
 *
 * @see       javax.servlet.http.HttpServletRequest
 *
 */
public class XHttpServletRequestFacade extends HttpServletRequestWrapper {
        
            
    public static final String DEFAULT_CHAR_ENCODING = "8859_1";

    /**
     * the inputstream to re-read.
     *
     * @see java.io.BufferedInputStream
     */
    private XServletInputStreamFacade inStreamFacade = null;
            
    /**
     * buffered reader which uses the input stream
     *
     * @see java.io.BufferedReader
     */
    private BufferedReader reader = null;
        
    /**
     * true - if inputstream can read using a reader; false otherwise.
     */
    private boolean usingReader = false;

    /**
     * true - if inputstream can read using a stream; false otherwise.
     */
    private boolean usingStream = false;
        
        
    /**
     * This constructor creates an re-readable HttpServletRequest.
     *
     * @pre        (req != null)
     * @post
     *
     * @param      req   HttpServletRequest
     *
     * @time
     * @space
     */
    public XHttpServletRequestFacade (HttpServletRequest request)  {
        super(request);
        Domain.debug("Create XHttpServletRequestFacade");
    }

    /**
     * Retrieves the body of the request as binary data using a ServletInputStream.
     *
     * @param      none
     *
     * @return     Returns the body of the request
     */
    public ServletInputStream getInputStream() throws IOException {
        Domain.debug("ENTER: XHttpServletRequestFacade:getInputStream()");
        if (usingReader) {
            throw new IllegalStateException("getReader() method has been called on this request");
        }
        usingStream = true;
        
        if ( inStreamFacade == null ) {
            inStreamFacade = new XServletInputStreamFacade( super.getInputStream());
        }
            
        Domain.debug("LEAVE: XHttpServletRequestFacade:getInputStream()");
        return (  inStreamFacade );
    }

        
    /**
     *  Reads the next byte of data from the input stream.
     *
     * @param      none
     *
     * @return     the next byte of data, or -1 if the end of the stream is reached
     */
    public int doRead() throws IOException {
        return inStreamFacade.read();
    }

    /**
     *  Reads up to len bytes of data from the input stream into an array of bytes.
     *
     * @param      b - the buffer into which the data is read.
     * @param      off - the start offset in array b at which the data is written.
     * @param      len - the maximum number of bytes to read.
     *
     * @return     the total number of bytes read into the buffer, or -1 if there is no more data because the end of the stream has been reached.
     */
    public int doRead( byte b[], int off, int len ) throws IOException {
        return inStreamFacade.read( b, off, len );
    }

        
    /**
     * Retrieves the body of the request as character data using a BufferedReader.
     *
     * @param      none
     *
     * @return     Returns a BufferedReader
     */
    public BufferedReader getReader() throws IOException {
        Domain.debug("ENTER: XHttpServletRequestFacade:getReader()");
        if (usingStream) {
            throw new IllegalStateException("getInputStream() method has been called on this request");
        }
        usingReader = true;

        if ( inStreamFacade == null ) {
            inStreamFacade = new XServletInputStreamFacade( super.getInputStream());
        }

        if( reader != null ) {
            Domain.debug("LEAVE: XHttpServletRequestFacade:getReader() - reader != null");
            return reader; // method already called
        }

        String encoding = super.getCharacterEncoding();
        if (encoding == null) {
            encoding = DEFAULT_CHAR_ENCODING;
        }
    
        InputStreamReader r = new InputStreamReader(inStreamFacade, encoding);
        reader= new BufferedReader(r);
        Domain.debug("LEAVE: XHttpServletRequestFacade:getReader() - new BufferedReader");
        return reader;
    }

    /**
     * Returns the content of the buffered input stream.
     *
     * @return      copy of input stream as string.
     */
    public String getRequestBody() throws IOException {
        if ( usingStream ){
            Domain.debug("XHttpServletRequestFacade:getRequestBody() - usingStream");
            return inStreamFacade.getBufferContent();
        } else if ( usingReader ) {
            Domain.debug("XHttpServletRequestFacade:getRequestBody() - usingReader");
            return reader.toString();
        } else {
            Domain.debug("XHttpServletRequestFacade:getRequestBody() - nor Reader nor Stream - do nothing");
            return "";
        }
    }
}
